import logging
from unittest import TestCase

from parameterized import parameterized, param

from hvac import exceptions
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestOkta(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "test-okta"
    TEST_ORG_NAME = "hvac-test"
    TEST_BASE_URL = "python-hvac.org"
    TEST_USERNAME = "hvac-person"
    TEST_GROUP = "hvac-group"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_auth_method(
            method_type="okta",
            path=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_auth_method(
            path=self.TEST_MOUNT_POINT,
        )
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_configure(self, label, raises=None, exception_msg=""):
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.configure(
                    org_name=self.TEST_ORG_NAME,
                    base_url=self.TEST_BASE_URL,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "not configured",
                configure_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_config(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)

        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.gcp.read_config(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            read_config_response = self.client.auth.okta.read_config(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % read_config_response)
            self.assertEqual(
                first=read_config_response["data"]["org_name"],
                second=self.TEST_ORG_NAME,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_list_users(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_user_response = self.client.auth.okta.register_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_user_response: %s" % register_user_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.list_users(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            list_users_response = self.client.auth.okta.list_users(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_users_response: %s" % list_users_response)
            self.assertEqual(
                first=list_users_response["data"]["keys"],
                second=[self.TEST_USERNAME],
            )

    @parameterized.expand(
        [
            param(
                "double register",
            ),
            param(
                "success",
                configure_first=False,
            ),
        ]
    )
    def test_register_user(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_user_response = self.client.auth.okta.register_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_user_response: %s" % register_user_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.register_user(
                    username=self.TEST_USERNAME,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            register_user_response = self.client.auth.okta.register_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_user_response: %s" % register_user_response)
            self.assertEqual(
                first=bool(register_user_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_user(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_user_response = self.client.auth.okta.register_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_user_response: %s" % register_user_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.read_user(
                    username=self.TEST_USERNAME,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            read_user_response = self.client.auth.okta.read_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_user_response: %s" % read_user_response)
            self.assertIn(
                member="policies",
                container=read_user_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
            ),
        ]
    )
    def test_delete_user(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_user_response = self.client.auth.okta.register_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_user_response: %s" % register_user_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.delete_user(
                    username=self.TEST_USERNAME,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            delete_user_response = self.client.auth.okta.delete_user(
                username=self.TEST_USERNAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_user_response: %s" % delete_user_response)
            self.assertEqual(
                first=bool(delete_user_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_list_groups(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_group_response = self.client.auth.okta.register_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_group_response: %s" % register_group_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.list_groups(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            list_groups_response = self.client.auth.okta.list_groups(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_groups_response: %s" % list_groups_response)
            self.assertEqual(
                first=list_groups_response["data"]["keys"],
                second=[self.TEST_GROUP],
            )

    @parameterized.expand(
        [
            param(
                "double register",
            ),
            param(
                "success",
                configure_first=False,
            ),
        ]
    )
    def test_register_group(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_group_response = self.client.auth.okta.register_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_group_response: %s" % register_group_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.register_group(
                    name=self.TEST_GROUP,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            register_group_response = self.client.auth.okta.register_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_group_response: %s" % register_group_response)
            self.assertEqual(
                first=bool(register_group_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_group(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_group_response = self.client.auth.okta.register_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_group_response: %s" % register_group_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.read_group(
                    name=self.TEST_GROUP,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            read_group_response = self.client.auth.okta.read_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_group_response: %s" % read_group_response)
            self.assertIn(
                member="policies",
                container=read_group_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no configuration",
                configure_first=False,
            ),
        ]
    )
    def test_delete_group(
        self, label, configure_first=True, raises=None, exception_msg=""
    ):
        if configure_first:
            configure_response = self.client.auth.okta.configure(
                org_name=self.TEST_ORG_NAME,
                base_url=self.TEST_BASE_URL,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            register_group_response = self.client.auth.okta.register_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("register_group_response: %s" % register_group_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.okta.delete_group(
                    name=self.TEST_GROUP,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            delete_group_response = self.client.auth.okta.delete_group(
                name=self.TEST_GROUP,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_group_response: %s" % delete_group_response)
            self.assertEqual(
                first=bool(delete_group_response),
                second=True,
            )



#!/usr/bin/env python
import json
import logging
from http.server import BaseHTTPRequestHandler


class MockOktaRequestHandler(BaseHTTPRequestHandler):
    """Small HTTP server used to mock out certain Okta API routes that vault requests in the Okta auth method."""

    def do_GET(self):
        """Dispatch GET requests to associated mock Okta 'handlers'."""
        self.send_response(200)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        logging.debug(self.headers)
        return
        if self.path == "/user":
            self.do_user()
        elif self.path == "/user/orgs?per_page=100":
            self.do_organizations_list()
        elif self.path == "/user/teams?per_page=100":
            self.do_team_list()
        return

    # def log_message(self, format, *args):
    #     """Squelch any HTTP logging."""
    #     return

    def do_user(self):
        """Return the bare minimum Okta user data needed for Vault's Okta auth method."""
        response = {
            "login": "hvac-dude",
            "id": 1,
        }

        self.wfile.write(json.dumps(response).encode())

    def do_organizations_list(self):
        """Return the bare minimum Okta organization data needed for Vault's Okta auth method.

        Only returns data if the request Authorization header has a contrived Okta token value of "valid-token".
        """
        response = []
        if self.headers.get("Authorization") == "Bearer valid-token":
            response.append(
                {
                    "login": "hvac",
                    "id": 1,
                }
            )

            self.wfile.write(json.dumps(response).encode())

    def do_team_list(self):
        """Return the bare minimum Okta team data needed for Vault's Okta auth method.

        Only returns data if the request Authorization header has a contrived Okta token value of "valid-token".
        """
        response = []
        if self.headers.get("Authorization") == "Bearer valid-token":
            response.append(
                {
                    "name": "hvac-team",
                    "slug": "hvac-team",
                    "organization": {
                        "id": 1,
                    },
                }
            )
        self.wfile.write(json.dumps(response).encode())



class VaultError(Exception):
    def __init__(
        self, message=None, errors=None, method=None, url=None, text=None, json=None
    ):
        if errors:
            message = ", ".join(errors)

        self.errors = errors
        self.method = method
        self.url = url
        self.text = text
        self.json = json

        super().__init__(message)

    def __str__(self):
        return f"{self.args[0]}, on {self.method} {self.url}"

    @classmethod
    def from_status(cls, status_code: int, *args, **kwargs):
        _STATUS_EXCEPTION_MAP = {
            400: InvalidRequest,
            401: Unauthorized,
            403: Forbidden,
            404: InvalidPath,
            429: RateLimitExceeded,
            500: InternalServerError,
            501: VaultNotInitialized,
            502: BadGateway,
            503: VaultDown,
        }

        return _STATUS_EXCEPTION_MAP.get(status_code, UnexpectedError)(*args, **kwargs)


class InvalidRequest(VaultError):
    pass


class Unauthorized(VaultError):
    pass


class Forbidden(VaultError):
    pass


class InvalidPath(VaultError):
    pass


class RateLimitExceeded(VaultError):
    pass


class InternalServerError(VaultError):
    pass


class VaultNotInitialized(VaultError):
    pass


class VaultDown(VaultError):
    pass


class UnexpectedError(VaultError):
    pass


class BadGateway(VaultError):
    pass


class ParamValidationError(VaultError):
    pass



from unittest import TestCase

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestSeal(HvacIntegrationTestCase, TestCase):
    def test_unseal_multi(self):
        cls = type(self)

        self.client.sys.seal()

        keys = cls.manager.keys

        result = self.client.sys.submit_unseal_keys(keys[0:2])

        self.assertTrue(result["sealed"])
        self.assertEqual(result["progress"], 2)

        result = self.client.sys.submit_unseal_key(reset=True)
        self.assertEqual(result["progress"], 0)
        result = self.client.sys.submit_unseal_keys(keys[1:3])
        self.assertTrue(result["sealed"])
        self.assertEqual(result["progress"], 2)
        self.client.sys.submit_unseal_keys(keys[0:1])
        result = self.client.sys.submit_unseal_keys(keys[2:3])
        self.assertFalse(result["sealed"])

    def test_seal_unseal(self):
        cls = type(self)

        self.assertFalse(self.client.sys.is_sealed())

        self.client.sys.seal()

        self.assertTrue(self.client.sys.is_sealed())

        cls.manager.unseal()

        self.assertFalse(self.client.sys.is_sealed())



import pytest

from unittest import mock
from hvac.api.auth_methods.token import Token
from hvac.adapters import Adapter


# TODO: move this to a conftest.py somewhere
class MockAdapter(Adapter):
    def __init__(self, *args, **kwargs):
        kwargs["session"] = mock.MagicMock()
        super().__init__(*args, **kwargs)

    def request(self, *args, **kwargs):
        return (args, kwargs)

    def get_login_token(self, response):
        raise NotImplementedError()


@pytest.fixture
def token_auth():
    return Token(MockAdapter())


class TestToken:
    @pytest.mark.parametrize("allowed_policies", ["allowed_policies", None])
    @pytest.mark.parametrize("disallowed_policies", ["disallowed_policies", None])
    @pytest.mark.parametrize("orphan", ["orphan", None])
    @pytest.mark.parametrize("renewable", ["renewable", None])
    @pytest.mark.parametrize("path_suffix", ["path_suffix", None])
    @pytest.mark.parametrize("allowed_entity_aliases", ["allowed_entity_aliases", None])
    @pytest.mark.parametrize("token_period", ["token_period", None])
    @pytest.mark.parametrize("token_explicit_max_ttl", ["token_explicit_max_ttl", None])
    def test_create_or_update_role_optional_parameters(
        self,
        token_auth,
        allowed_policies,
        disallowed_policies,
        orphan,
        renewable,
        path_suffix,
        allowed_entity_aliases,
        token_period,
        token_explicit_max_ttl,
    ):
        params = {
            "allowed_policies": allowed_policies,
            "disallowed_policies": disallowed_policies,
            "orphan": orphan,
            "renewable": renewable,
            "path_suffix": path_suffix,
            "allowed_entity_aliases": allowed_entity_aliases,
            "token_period": token_period,
            "token_explicit_max_ttl": token_explicit_max_ttl,
        }
        expected = params.copy()

        _, rkwargs = token_auth.create_or_update_role("role_name", **params)

        assert "json" in rkwargs
        for key, value in expected.items():
            assert value is None or rkwargs["json"][key] == value



from unittest import TestCase

from parameterized import parameterized

from hvac import exceptions
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestKvV1(HvacIntegrationTestCase, TestCase):
    DEFAULT_MOUNT_POINT = "kvv1"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="kv",
            path=self.DEFAULT_MOUNT_POINT,
            options=dict(version=1),
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.DEFAULT_MOUNT_POINT)
        super().tearDown()

    @parameterized.expand(
        [
            ("nonexistent secret", "no-secret-here", False, exceptions.InvalidPath),
            ("read secret", "top-secret"),
        ]
    )
    def test_read_secret(
        self,
        test_label,
        path,
        write_secret_before_test=True,
        raises=None,
        exception_message="",
    ):
        test_secret = {
            "pssst": "hi",
        }
        if write_secret_before_test:
            self.client.secrets.kv.v1.create_or_update_secret(
                path=path,
                secret=test_secret,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.kv.v1.read_secret(
                    path=path,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:

            read_secret_result = self.client.secrets.kv.v1.read_secret(
                path=path,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
            self.assertDictEqual(
                d1=test_secret,
                d2=read_secret_result["data"],
            )

    @parameterized.expand(
        [
            (
                "nonexistent secret",
                "hvac/no-secret-here",
                False,
                exceptions.InvalidPath,
            ),
            ("list secret", "hvac/top-secret"),
        ]
    )
    def test_list_secrets(
        self,
        test_label,
        path,
        write_secret_before_test=True,
        raises=None,
        exception_message="",
    ):
        test_secret = {
            "pssst": "hi",
        }
        test_path_prefix, test_key = path.split("/")[:2]

        if write_secret_before_test:
            self.client.secrets.kv.v1.create_or_update_secret(
                path=path,
                secret=test_secret,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.kv.v1.list_secrets(
                    path=test_path_prefix,
                    mount_point=self.DEFAULT_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            list_secrets_result = self.client.secrets.kv.v1.list_secrets(
                path=test_path_prefix,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
            self.assertEqual(
                first=dict(keys=[test_key]),
                second=list_secrets_result["data"],
            )

    @parameterized.expand(
        [
            ("create secret no method specified", "hvac", None, False),
            ("create secret post method specified", "hvac", "POST", False),
            (
                "create secret invalid method specified",
                "hvac",
                "GET",
                False,
                exceptions.ParamValidationError,
                '"method" parameter provided invalid value',
            ),
            ("update secret no method specified", "hvac", None),
            ("update secret put method specified", "hvac", "PUT"),
            (
                "update secret invalid method specified",
                "hvac",
                "GET",
                True,
                exceptions.ParamValidationError,
                '"method" parameter provided invalid value',
            ),
        ]
    )
    def test_create_or_update_secret(
        self,
        test_label,
        path,
        method=None,
        write_secret_before_test=True,
        raises=None,
        exception_message="",
    ):
        test_secret = {
            "pssst": "hi",
        }

        if write_secret_before_test:
            self.client.secrets.kv.v1.create_or_update_secret(
                path=path,
                secret=test_secret,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.kv.v1.create_or_update_secret(
                    path=path,
                    secret=test_secret,
                    method=method,
                    mount_point=self.DEFAULT_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            create_or_update_secret_result = (
                self.client.secrets.kv.v1.create_or_update_secret(
                    path=path,
                    secret=test_secret,
                    method=method,
                    mount_point=self.DEFAULT_MOUNT_POINT,
                )
            )
            self.assertEqual(
                first=204,
                second=create_or_update_secret_result.status_code,
            )

    @parameterized.expand(
        [
            ("nonexistent secret", "hvac/no-secret-here"),
            ("delete secret", "hvac/top-secret"),
        ]
    )
    def test_delete_secret(
        self,
        test_label,
        path,
        write_secret_before_test=True,
        raises=None,
        exception_message="",
    ):
        test_secret = {
            "pssst": "hi",
        }

        if write_secret_before_test:
            self.client.secrets.kv.v1.create_or_update_secret(
                path=path,
                secret=test_secret,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.kv.v1.delete_secret(
                    path=path,
                    mount_point=self.DEFAULT_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            delete_secret_result = self.client.secrets.kv.v1.delete_secret(
                path=path,
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
            self.assertEqual(
                first=204,
                second=delete_secret_result.status_code,
            )



import logging
from unittest import TestCase
from urllib.parse import parse_qs, urlparse

from parameterized import param, parameterized
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase
from tests.utils.mock_oauth_provider.mock_oauth_provider import (
    MockOauthProviderServerThread,
    create_user_session_and_client,
)


class TestOIDC(HvacIntegrationTestCase, TestCase):
    TEST_APPROLE_PATH = "oidc-test-approle"
    TEST_APPROLE_ROLE_ID = "oidc-test-role-id"
    TEST_OIDC_PATH = "oidc-test"
    oidc_client_id = "hvac-client-id"
    oidc_server = None

    def setUp(self):
        super().setUp()
        self.client.sys.enable_auth_method(
            method_type="oidc",
            path=self.TEST_OIDC_PATH,
        )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_OIDC_PATH,
        )
        self.client.sys.disable_auth_method(
            path=self.TEST_APPROLE_PATH,
        )
        if self.oidc_server:
            self.oidc_server.shutdown()
            self.oidc_server = None

    @parameterized.expand(
        [
            param(
                "configure using vault identity OIDC",
                issuer="https://localhost:8200",
            ),
        ]
    )
    def test_configure(self, label, issuer):
        oidc_discovery_url = f"{issuer}/v1/identity/oidc"
        self.client.secrets.identity.configure_tokens_backend(
            issuer=issuer,
        )
        response = self.client.auth.oidc.configure(
            oidc_discovery_url=oidc_discovery_url,
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("configure response: %s" % response)
        self.assertEqual(
            first=204,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            param(
                "configure using vault identity OIDC",
                issuer="https://localhost:8200",
            ),
        ]
    )
    def test_read_config(self, label, issuer):
        oidc_discovery_url = f"{issuer}/v1/identity/oidc"
        self.client.secrets.identity.configure_tokens_backend(
            issuer=issuer,
        )
        configure_response = self.client.auth.oidc.configure(
            oidc_discovery_url=oidc_discovery_url,
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("configure response: %s" % configure_response)
        response = self.client.auth.oidc.read_config(
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("read_config response: %s" % response)
        self.assertEqual(
            first=oidc_discovery_url,
            second=response["data"]["oidc_discovery_url"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/oidc-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_read_role(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.oidc.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.oidc.read_role(
            name=role_name,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("read_role response: %s" % response)
        self.assertEqual(
            first=user_claim,
            second=response["data"]["user_claim"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/oidc-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_list_roles(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.oidc.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.oidc.list_roles(
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("list_roles response: %s" % response)
        self.assertIn(
            member=role_name,
            container=response["data"]["keys"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/oidc-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_delete_role(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.oidc.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.oidc.delete_role(
            name=role_name,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("delete_role response: %s" % response)
        self.assertEqual(
            first=204,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            param(
                "success",
                issuer="https://localhost:8200",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/oidc-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_oidc_authorization_url_request(
        self, label, issuer, role_name, allowed_redirect_uris, user_claim
    ):
        if "%s/" % self.TEST_APPROLE_PATH not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="approle",
                path=self.TEST_APPROLE_PATH,
            )
        id_token_role_name = "hvac-oidc-test"
        key_name = "oidc-test-key"
        create_named_key_response = self.client.secrets.identity.create_named_key(
            name=key_name,
        )
        create_or_update_role_response = (
            self.client.secrets.identity.create_or_update_role(
                name=id_token_role_name,
                key=key_name,
            )
        )
        logging.debug(
            "create_or_update_role response: %s" % create_or_update_role_response
        )
        read_role_response = self.client.secrets.identity.read_role(
            name=id_token_role_name,
        )
        logging.debug("read_role response: %s" % read_role_response)
        token_client_id = read_role_response["data"]["client_id"]
        create_named_key_response = self.client.secrets.identity.create_named_key(
            name=key_name,
            allowed_client_ids=[
                token_client_id,
            ],
        )
        logging.debug("create_named_key response: %s" % create_named_key_response)
        # Log in using a dummy approle role so our client token has an associated identity
        self.login_using_admin_approle_role(
            role_id=self.TEST_APPROLE_ROLE_ID,
            path=self.TEST_APPROLE_PATH,
        )
        generate_token_response = self.client.secrets.identity.generate_signed_id_token(
            name=id_token_role_name,
        )
        logging.debug("generate_token_response: %s" % generate_token_response)

        oidc_discovery_url = f"{issuer}/v1/identity/oidc"
        self.client.secrets.identity.configure_tokens_backend(
            issuer=issuer,
        )
        response = self.client.auth.oidc.configure(
            oidc_discovery_url=oidc_discovery_url,
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            oidc_client_id=self.oidc_client_id,
            oidc_client_secret=generate_token_response["data"]["token"],
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("configure response: %s" % response)
        create_role_response = self.client.auth.oidc.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.oidc.oidc_authorization_url_request(
            role=role_name,
            redirect_uri=allowed_redirect_uris[0],
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("oidc_authorization_url_request response: %s" % response)
        self.assertIn(
            member=f"?client_id={self.oidc_client_id}",
            container=response["data"]["auth_url"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac-oidc",
                allowed_redirect_uris=[
                    "https://localhost:8200/v1/auth/oidc-test/oidc/callback"
                ],
                user_claim="sub",
            ),
        ]
    )
    def test_oidc_callback(self, label, role_name, allowed_redirect_uris, user_claim):
        self.oidc_server = MockOauthProviderServerThread()
        self.oidc_server.start()
        oidc_details = create_user_session_and_client(
            server_url=self.oidc_server.url,
            oauth_redirect_uri=allowed_redirect_uris[0],
        )

        response = self.client.auth.oidc.configure(
            oidc_discovery_url=oidc_details["discovery_url"],
            oidc_response_mode="form_post",
            oidc_response_types=["code"],
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            oidc_client_id=oidc_details["client_id"],
            oidc_client_secret=oidc_details["client_secret"],
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("oidc.configure response: %s" % response)

        create_role_response = self.client.auth.oidc.create_role(
            name=role_name,
            role_type="oidc",
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            verbose_oidc_logging=True,
            path=self.TEST_OIDC_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)

        oidc_authorization_url_response = (
            self.client.auth.oidc.oidc_authorization_url_request(
                role=role_name,
                redirect_uri=allowed_redirect_uris[0],
                path=self.TEST_OIDC_PATH,
            )
        )
        logging.debug(
            "oidc_authorization_url_request response: %s"
            % oidc_authorization_url_response
        )

        auth_url = oidc_authorization_url_response["data"]["auth_url"]
        logging.debug("auth_url: %s" % auth_url)
        auth_url_qs = urlparse(auth_url).query
        auth_url_qs_parsed = parse_qs(auth_url_qs)
        logging.debug("auth_url_qs_parsed: %s" % auth_url_qs_parsed)

        authorize_response = oidc_details["session"].get(
            url=auth_url,
        )
        logging.debug("authorize_response: %s" % authorize_response.json())
        authorization = authorize_response.json()
        logging.debug("authorization: %s" % authorization)

        client_cert_path = utils.get_config_file_path("client-cert.pem")
        client_key_path = utils.get_config_file_path("client-key.pem")
        server_cert_path = utils.get_config_file_path("server-cert.pem")
        response = oidc_details["session"].post(
            url=auth_url,
            cert=(client_cert_path, client_key_path),
            verify=server_cert_path,
            data=dict(confirm=True),
        )
        oidc_auth_data = response.json()
        logging.debug("oidc_auth_data: %s" % oidc_auth_data)
        self.client.token = oidc_auth_data["auth"]["client_token"]
        self.assertIn(
            member=role_name,
            container=self.client.lookup_token()["data"]["meta"]["role"],
        )



import logging
from unittest import TestCase

from parameterized import param, parameterized

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestGcp(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "test-gcp"
    TEST_ROLESET_NAME = "hvac-roleset"
    TEST_PROJECT_ID = "test-hvac"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="gcp",
            path=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.TEST_MOUNT_POINT)
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_write_config(
        self, label, max_ttl=3600, raises=False, exception_message=""
    ):
        credentials = utils.load_config_file("example.jwt.json")
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.gcp.configure(
                    credentials=credentials,
                    max_ttl=max_ttl,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            configure_response = self.client.secrets.gcp.configure(
                credentials=credentials,
                max_ttl=max_ttl,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )
            read_configuration_response = self.client.secrets.gcp.read_config(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug(
                "read_configuration_response: %s" % read_configuration_response
            )
            self.assertEqual(
                first=read_configuration_response["data"]["max_ttl"],
                second=max_ttl,
            )



import logging
from unittest import TestCase
from unittest import skipIf

from parameterized import parameterized, param

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(
    utils.vault_version_lt("0.8.3"),
    "Kubernetes auth method not available before Vault version 0.8.3",
)
class TestKubernetes(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "kubernetes-test"

    def setUp(self):
        super().setUp()
        if "%s/" % self.TEST_MOUNT_POINT not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="kubernetes",
                path=self.TEST_MOUNT_POINT,
            )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_MOUNT_POINT,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param("issuer test", issuer="bob"),
            param(
                "set invalid kubernetes_ca_cert",
                kubernetes_ca_cert="ca_cert",
                raises=exceptions.ParamValidationError,
                exception_message="required type: PEM",
            ),
            param(
                "set invalid pem_key",
                kubernetes_ca_cert="-----BEGIN CERTIFICATE-----\\n.....\\n-----END CERTIFICATE-----",
                pem_keys=["pem_key"],
                raises=exceptions.ParamValidationError,
                exception_message="required type: PEM",
            ),
            param(
                "set invalid token_reviewer_jwt",
                kubernetes_ca_cert="-----BEGIN CERTIFICATE-----\\n.....\\n-----END CERTIFICATE-----",
                token_reviewer_jwt="reviewer_jwt",
                issuer="bob",
                raises=exceptions.InternalServerError,
                exception_message="* not a compact JWS"
                if utils.vault_version_lt("1.11.0")
                else "compact JWS format must have three parts",
            ),
            param(
                "missing kubernetes_ca_cert",
                disable_local_ca_jwt=True,
                raises=exceptions.InvalidRequest,
                exception_message="one of pem_keys or kubernetes_ca_cert must be set"
                if utils.vault_version_lt(
                    "1.9.5"
                )  # guessing based on 1.9.4 changelog touching kubernetes auth
                else "kubernetes_ca_cert must be given",
            ),
        ]
    )
    def test_configure(
        self,
        label,
        kubernetes_ca_cert=None,
        token_reviewer_jwt=None,
        pem_keys=None,
        issuer=None,
        disable_local_ca_jwt=False,
        raises=None,
        exception_message="",
    ):
        kubernetes_host = "https://192.168.99.100:8443"
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.kubernetes.configure(
                    kubernetes_host=kubernetes_host,
                    kubernetes_ca_cert=kubernetes_ca_cert,
                    token_reviewer_jwt=token_reviewer_jwt,
                    pem_keys=pem_keys,
                    mount_point=self.TEST_MOUNT_POINT,
                    disable_local_ca_jwt=disable_local_ca_jwt,
                )
            self.assertIn(member=exception_message, container=str(cm.exception))
        else:
            configure_response = self.client.auth.kubernetes.configure(
                kubernetes_host=kubernetes_host,
                kubernetes_ca_cert="-----BEGIN CERTIFICATE-----\\n.....\\n-----END CERTIFICATE-----",
                mount_point=self.TEST_MOUNT_POINT,
                issuer="bob",
                disable_local_ca_jwt=disable_local_ca_jwt,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no config written yet",
                write_config_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_config(self, label, write_config_first=True, raises=None):
        expected_config = {
            "kubernetes_host": "https://192.168.99.100:8443",
            "kubernetes_ca_cert": "-----BEGIN CERTIFICATE-----\\n.....\\n-----END CERTIFICATE-----",
        }
        if write_config_first:
            self.client.auth.kubernetes.configure(
                mount_point=self.TEST_MOUNT_POINT, **expected_config
            )
        if raises is not None:
            with self.assertRaises(raises):
                self.client.auth.kubernetes.read_config(
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            read_config_response = self.client.auth.kubernetes.read_config(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_config_response: %s" % read_config_response)
            for k, v in expected_config.items():
                self.assertEqual(
                    first=v,
                    second=read_config_response[k],
                )

    @parameterized.expand(
        [
            param(
                "success",
                bound_service_account_names=["vault-auth"],
                bound_service_account_namespaces=["default"],
            ),
            param(
                "both bounds wildcard permitted",
                bound_service_account_names=["*"],
                bound_service_account_namespaces=["*"],
            ),
            param(
                "token type",
                bound_service_account_names=["vault-auth"],
                bound_service_account_namespaces=["default"],
                token_type="service",
            ),
            param(
                "serviceaccount name for alias",
                bound_service_account_names=["vault-auth"],
                bound_service_account_namespaces=["default"],
                alias_name_source="serviceaccount_name",
                token_type="service",
            ),
        ]
    )
    def test_create_role(
        self,
        label,
        bound_service_account_names=None,
        bound_service_account_namespaces=None,
        token_type=None,
        alias_name_source=None,
        raises=None,
        exception_message="",
    ):
        role_name = "test-role"
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.kubernetes.create_role(
                    name=role_name,
                    bound_service_account_names=bound_service_account_names,
                    bound_service_account_namespaces=bound_service_account_namespaces,
                    token_type=token_type,
                    alias_name_source=alias_name_source,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            create_role_response = self.client.auth.kubernetes.create_role(
                name=role_name,
                bound_service_account_names=bound_service_account_names,
                bound_service_account_namespaces=bound_service_account_namespaces,
                token_type=token_type,
                alias_name_source=alias_name_source,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)
            self.assertEqual(
                first=bool(create_role_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "nonexistent role name",
                create_role_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_role(
        self, label, create_role_first=True, raises=None, exception_message=""
    ):
        role_name = "test-role"
        expected_role_config = {
            "name": role_name,
            "bound_service_account_names": ["vault-auth"],
            "bound_service_account_namespaces": ["default"],
        }
        role_name = "test-role"

        if create_role_first:
            self.client.auth.kubernetes.create_role(
                mount_point=self.TEST_MOUNT_POINT, **expected_role_config
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.kubernetes.read_role(
                    name=role_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            read_role_response = self.client.auth.kubernetes.read_role(
                name=role_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_role_response: %s" % read_role_response)
            self.assertEqual(
                first=read_role_response["bound_service_account_names"],
                second=expected_role_config["bound_service_account_names"],
            )
            self.assertEqual(
                first=read_role_response["bound_service_account_namespaces"],
                second=expected_role_config["bound_service_account_namespaces"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "no roles",
                num_roles_to_create=0,
                raises=exceptions.InvalidPath,
            ),
            param(
                "no config",
                write_config_first=False,
            ),
        ]
    )
    def test_list_roles(
        self, label, num_roles_to_create=1, write_config_first=True, raises=None
    ):
        if write_config_first:
            self.client.auth.kubernetes.configure(
                kubernetes_host="https://192.168.99.100:8443",
                kubernetes_ca_cert="-----BEGIN CERTIFICATE-----\n.....\n-----END CERTIFICATE-----",
                mount_point=self.TEST_MOUNT_POINT,
            )
        roles_to_create = [f"hvac{str(n)}" for n in range(0, num_roles_to_create)]
        bound_service_account_names = ["vault-auth"]
        bound_service_account_namespaces = ["default"]
        logging.debug("roles_to_create: %s" % roles_to_create)
        for role_to_create in roles_to_create:
            create_role_response = self.client.auth.kubernetes.create_role(
                name=role_to_create,
                bound_service_account_names=bound_service_account_names,
                bound_service_account_namespaces=bound_service_account_namespaces,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)

        if raises:
            with self.assertRaises(raises):
                self.client.auth.kubernetes.list_roles(
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            list_roles_response = self.client.auth.kubernetes.list_roles(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_roles_response: %s" % list_roles_response)
            self.assertEqual(
                first=list_roles_response["keys"],
                second=roles_to_create,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "nonexistent role name",
                configure_role_first=False,
            ),
        ]
    )
    def test_delete_role(self, label, configure_role_first=True, raises=None):
        role_name = "test-role"
        bound_service_account_names = ["vault-auth"]
        bound_service_account_namespaces = ["default"]
        if configure_role_first:
            create_role_response = self.client.auth.kubernetes.create_role(
                name=role_name,
                bound_service_account_names=bound_service_account_names,
                bound_service_account_namespaces=bound_service_account_namespaces,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)

        if raises is not None:
            with self.assertRaises(raises):
                self.client.auth.kubernetes.delete_role(
                    name=role_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            delete_role_response = self.client.auth.kubernetes.delete_role(
                name=role_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_role_response: %s" % delete_role_response)
            self.assertEqual(
                first=bool(delete_role_response),
                second=True,
            )



#!/usr/bin/env python
"""KvV1 methods module."""
from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "secret"


class KvV1(VaultApiBase):
    """KV Secrets Engine - Version 1 (API).

    Reference: https://www.vaultproject.io/api/secrets/kv/kv-v1.html
    """

    def read_secret(self, path, mount_point=DEFAULT_MOUNT_POINT):
        """Retrieve the secret at the specified location.

        Supported methods:
            GET: /{mount_point}/{path}. Produces: 200 application/json


        :param path: Specifies the path of the secret to read. This is specified as part of the URL.
        :type path: str | unicode
        :param mount_point: The "path" the secret engine was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_secret request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/{path}", mount_point=mount_point, path=path
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_secrets(self, path, mount_point=DEFAULT_MOUNT_POINT):
        """Return a list of key names at the specified location.

        Folders are suffixed with /. The input must be a folder; list on a file will not return a value. Note that no
        policy-based filtering is performed on keys; do not encode sensitive information in key names. The values
        themselves are not accessible via this command.

        Supported methods:
            LIST: /{mount_point}/{path}. Produces: 200 application/json

        :param path: Specifies the path of the secrets to list.
            This is specified as part of the URL.
        :type path: str | unicode
        :param mount_point: The "path" the secret engine was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the list_secrets request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/{path}", mount_point=mount_point, path=path
        )
        return self._adapter.list(
            url=api_path,
        )

    def create_or_update_secret(
        self, path, secret, method=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Store a secret at the specified location.

        If the value does not yet exist, the calling token must have an ACL policy granting the create capability.
        If the value already exists, the calling token must have an ACL policy granting the update capability.

        Supported methods:
            POST: /{mount_point}/{path}. Produces: 204 (empty body)
            PUT: /{mount_point}/{path}. Produces: 204 (empty body)

        :param path: Specifies the path of the secrets to create/update. This is specified as part of the URL.
        :type path: str | unicode
        :param secret: Specifies keys, paired with associated values, to be held at the given location. Multiple
            key/value pairs can be specified, and all will be returned on a read operation. A key called ttl will
            trigger some special behavior. See the Vault KV secrets engine documentation for details.
        :type secret: dict
        :param method: Optional parameter to explicitly request a POST (create) or PUT (update) request to the selected
            kv secret engine. If no argument is provided for this parameter, hvac attempts to intelligently determine
            which method is appropriate.
        :type method: str | unicode
        :param mount_point: The "path" the secret engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_secret request.
        :rtype: requests.Response
        """
        if method is None:
            # If no method was selected by the caller, use the result of a `read_secret()` call to determine if we need
            # to perform an update (PUT) or creation (POST) request.
            try:
                self.read_secret(
                    path=path,
                    mount_point=mount_point,
                )
                method = "PUT"
            except exceptions.InvalidPath:
                method = "POST"

        if method == "POST":
            api_path = utils.format_url(
                "/v1/{mount_point}/{path}", mount_point=mount_point, path=path
            )
            return self._adapter.post(
                url=api_path,
                json=secret,
            )

        elif method == "PUT":
            api_path = utils.format_url(
                "/v1/{mount_point}/{path}", mount_point=mount_point, path=path
            )
            return self._adapter.put(
                url=api_path,
                json=secret,
            )

        else:
            error_message = '"method" parameter provided invalid value; POST or PUT allowed, "{method}" provided'.format(
                method=method
            )
            raise exceptions.ParamValidationError(error_message)

    def delete_secret(self, path, mount_point=DEFAULT_MOUNT_POINT):
        """Delete the secret at the specified location.

        Supported methods:
            DELETE: /{mount_point}/{path}. Produces: 204 (empty body)


        :param path: Specifies the path of the secret to delete.
            This is specified as part of the URL.
        :type path: str | unicode
        :param mount_point: The "path" the secret engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_secret request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/{path}", mount_point=mount_point, path=path
        )
        return self._adapter.delete(
            url=api_path,
        )



"""Collection of Vault system backend API endpoint classes."""
import logging

from hvac.api.system_backend.audit import Audit
from hvac.api.system_backend.auth import Auth
from hvac.api.system_backend.capabilities import Capabilities
from hvac.api.system_backend.health import Health
from hvac.api.system_backend.init import Init
from hvac.api.system_backend.key import Key
from hvac.api.system_backend.leader import Leader
from hvac.api.system_backend.lease import Lease
from hvac.api.system_backend.mount import Mount
from hvac.api.system_backend.namespace import Namespace
from hvac.api.system_backend.policy import Policy
from hvac.api.system_backend.raft import Raft
from hvac.api.system_backend.seal import Seal
from hvac.api.system_backend.wrapping import Wrapping
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from hvac.api.vault_api_category import VaultApiCategory

__all__ = (
    "Audit",
    "Auth",
    "Capabilities",
    "Health",
    "Init",
    "Key",
    "Leader",
    "Lease",
    "Mount",
    "Namespace",
    "Policy",
    "Raft",
    "Seal",
    "SystemBackend",
    "SystemBackendMixin",
    "Wrapping",
)


logger = logging.getLogger(__name__)


class SystemBackend(
    VaultApiCategory,
    Audit,
    Auth,
    Capabilities,
    Health,
    Init,
    Key,
    Leader,
    Lease,
    Mount,
    Namespace,
    Policy,
    Raft,
    Seal,
    Wrapping,
):
    implemented_classes = [
        Audit,
        Auth,
        Capabilities,
        Health,
        Init,
        Key,
        Leader,
        Lease,
        Mount,
        Namespace,
        Policy,
        Raft,
        Seal,
        Wrapping,
    ]
    unimplemented_classes = []



import logging
from unittest import TestCase
from unittest import skipIf

from parameterized import parameterized, param

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestTransit(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "transit-integration-test"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="transit",
            path=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.TEST_MOUNT_POINT)
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param("auto rotate period field", auto_rotate_period="1h"),
        ]
    )
    def test_create_key(
        self, label, raises=False, auto_rotate_period=None, exception_message=""
    ):
        key_name = "testkey"
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.create_key(
                    name=key_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            create_key_response = self.client.secrets.transit.create_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_key_response: %s" % create_key_response)
            self.assertEqual(
                first=bool(create_key_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_key(self, label, raises=False, exception_message=""):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.read_key(
                    name=key_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            read_key_response = self.client.secrets.transit.read_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_key_response: %s" % read_key_response)
            self.assertEqual(
                first=read_key_response["data"]["name"],
                second=key_name,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_keys(self, label, raises=False, exception_message=""):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.list_keys(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            list_keys_response = self.client.secrets.transit.list_keys(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_keys_response: %s" % list_keys_response)
            self.assertEqual(
                first=list_keys_response["data"]["keys"],
                second=[key_name],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_key(self, label, raises=False, exception_message=""):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        update_key_configuration_response = (
            self.client.secrets.transit.update_key_configuration(
                name=key_name,
                deletion_allowed=True,
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "update_key_configuration_response: %s" % update_key_configuration_response
        )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.delete_key(
                    name=key_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            delete_key_response = self.client.secrets.transit.delete_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_key_response: %s" % delete_key_response)
            self.assertEqual(
                first=bool(delete_key_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_rotate_key(self, label, raises=False, exception_message=""):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.rotate_key(
                    name=key_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            rotate_key_response = self.client.secrets.transit.rotate_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("rotate_key_response: %s" % rotate_key_response)
            self.assertEqual(
                first=bool(rotate_key_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid key type",
                key_type="kitty-cat-key",
                raises=exceptions.ParamValidationError,
                exception_message="invalid key_type argument provided",
            ),
        ]
    )
    def test_export_key(
        self, label, key_type="hmac-key", raises=False, exception_message=""
    ):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            exportable=True,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.export_key(
                    name=key_name,
                    key_type=key_type,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            export_key_response = self.client.secrets.transit.export_key(
                name=key_name,
                key_type=key_type,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("export_key_response: %s" % export_key_response)
            self.assertEqual(
                first=len(export_key_response["data"]["keys"]),
                second=1,
            )
            self.assertEqual(
                first=export_key_response["data"]["name"],
                second=key_name,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_encrypt_data(
        self, label, plaintext="hi itsame hvac", raises=False, exception_message=""
    ):
        key_name = "testkey"
        plaintext = utils.base64ify(plaintext)
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.encrypt_data(
                    name=key_name,
                    plaintext=plaintext,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            encrypt_data_response = self.client.secrets.transit.encrypt_data(
                name=key_name,
                plaintext=plaintext,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("encrypt_data_response: %s" % encrypt_data_response)
            self.assertIn(
                member="ciphertext",
                container=encrypt_data_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_decrypt_data(
        self, label, plaintext="hi itsame hvac", raises=False, exception_message=""
    ):
        key_name = "testkey"
        plaintext = utils.base64ify(plaintext)
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        encrypt_data_response = self.client.secrets.transit.encrypt_data(
            name=key_name,
            plaintext=plaintext,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("encrypt_data_response: %s" % encrypt_data_response)
        ciphertext = encrypt_data_response["data"]["ciphertext"]
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.decrypt_data(
                    name=key_name,
                    ciphertext=ciphertext,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            decrypt_data_response = self.client.secrets.transit.decrypt_data(
                name=key_name,
                ciphertext=ciphertext,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("decrypt_data_response: %s" % decrypt_data_response)
            self.assertIn(
                member=plaintext,
                container=decrypt_data_response["data"]["plaintext"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_rewrap_data(
        self, label, plaintext="hi itsame hvac", raises=False, exception_message=""
    ):
        key_name = "testkey"
        plaintext = utils.base64ify(plaintext)
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        encrypt_data_response = self.client.secrets.transit.encrypt_data(
            name=key_name,
            plaintext=plaintext,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("encrypt_data_response: %s" % encrypt_data_response)
        ciphertext = encrypt_data_response["data"]["ciphertext"]
        rotate_key_response = self.client.secrets.transit.rotate_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("rotate_key_response: %s" % rotate_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.rewrap_data(
                    name=key_name,
                    ciphertext=ciphertext,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            rewrap_data_response = self.client.secrets.transit.rewrap_data(
                name=key_name,
                ciphertext=ciphertext,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("rewrap_data_response: %s" % rewrap_data_response)
            self.assertIn(
                member="ciphertext",
                container=rewrap_data_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid key type",
                key_type="kitty-cat-key",
                raises=exceptions.ParamValidationError,
                exception_message="invalid key_type argument provided",
            ),
        ]
    )
    def test_generate_data_key(
        self, label, key_type="plaintext", raises=False, exception_message=""
    ):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.generate_data_key(
                    name=key_name,
                    key_type=key_type,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            gen_data_key_response = self.client.secrets.transit.generate_data_key(
                name=key_name,
                key_type=key_type,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("gen_data_key_response: %s" % gen_data_key_response)
            self.assertIn(
                member="ciphertext",
                container=gen_data_key_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_generate_random_bytes(
        self, label, n_bytes=32, raises=False, exception_message=""
    ):
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.generate_random_bytes(
                    n_bytes=n_bytes,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            gen_bytes_response = self.client.secrets.transit.generate_random_bytes(
                n_bytes=n_bytes,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("gen_data_key_response: %s" % gen_bytes_response)
            self.assertIn(
                member="random_bytes",
                container=gen_bytes_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid algorithm",
                algorithm="meow2-256",
                raises=exceptions.ParamValidationError,
                exception_message="invalid algorithm argument provided",
            ),
            param(
                "invalid output_format",
                output_format="kitty64",
                raises=exceptions.ParamValidationError,
                exception_message="invalid output_format argument provided",
            ),
        ]
    )
    def test_hash_data(
        self,
        label,
        hash_input="hash this ish",
        algorithm="sha2-256",
        output_format="hex",
        raises=False,
        exception_message="",
    ):
        hash_input = utils.base64ify(hash_input)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.hash_data(
                    hash_input=hash_input,
                    algorithm=algorithm,
                    output_format=output_format,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            hash_data_response = self.client.secrets.transit.hash_data(
                hash_input=hash_input,
                algorithm=algorithm,
                output_format=output_format,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("hash_data_response: %s" % hash_data_response)
            self.assertIn(
                member="sum",
                container=hash_data_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid algorithm",
                algorithm="meow2-256",
                raises=exceptions.ParamValidationError,
                exception_message="invalid algorithm argument provided",
            ),
        ]
    )
    def test_generate_hmac(
        self,
        label,
        hash_input="hash this ish",
        algorithm="sha2-256",
        raises=False,
        exception_message="",
    ):
        hash_input = utils.base64ify(hash_input)
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.generate_hmac(
                    name=key_name,
                    hash_input=hash_input,
                    algorithm=algorithm,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            generate_hmac_response = self.client.secrets.transit.generate_hmac(
                name=key_name,
                hash_input=hash_input,
                algorithm=algorithm,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("generate_hmac_response: %s" % generate_hmac_response)
            self.assertIn(
                member="hmac",
                container=generate_hmac_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid algorithm",
                hash_algorithm="meow2-256",
                raises=exceptions.ParamValidationError,
                exception_message="invalid hash_algorithm argument provided",
            ),
            param(
                "invalid signature_algorithm",
                signature_algorithm="pre-shared kitty cats",
                raises=exceptions.ParamValidationError,
                exception_message="invalid signature_algorithm argument provided",
            ),
            param(
                "auto salt length",
                salt_length="auto",
            ),
            param(
                "hash salt length",
                salt_length="hash",
            ),
            param(
                "invalid string for salt length",
                salt_length="foobar",
                raises=exceptions.ParamValidationError,
                exception_message="invalid salt_length argument provided",
            ),
            param(
                "invalid negative salt length",
                salt_length="-2",
                raises=exceptions.ParamValidationError,
                exception_message="invalid salt_length argument provided",
            ),
            param(
                "valid negative salt length",
                salt_length="-1",
            ),
            param(
                "zero salt length",
                salt_length="0",
            ),
            param(
                "positive salt length",
                salt_length="1",
            ),
        ]
    )
    def test_sign_data(
        self,
        label,
        hash_input="hash this ish",
        hash_algorithm="sha2-256",
        signature_algorithm="pss",
        salt_length=None,
        raises=False,
        exception_message="",
    ):
        hash_input = utils.base64ify(hash_input)
        key_name = "testkey"
        key_type = "ed25519"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.sign_data(
                    name=key_name,
                    hash_input=hash_input,
                    hash_algorithm=hash_algorithm,
                    signature_algorithm=signature_algorithm,
                    salt_length=salt_length,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            sign_data_response = self.client.secrets.transit.sign_data(
                name=key_name,
                hash_input=hash_input,
                hash_algorithm=hash_algorithm,
                signature_algorithm=signature_algorithm,
                salt_length=salt_length,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("sign_data_response: %s" % sign_data_response)
            self.assertIn(
                member="signature",
                container=sign_data_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid algorithm",
                hash_algorithm="meow2-256",
                raises=exceptions.ParamValidationError,
                exception_message="invalid hash_algorithm argument provided",
            ),
            param(
                "invalid signature_algorithm",
                signature_algorithm="pre-shared kitty cats",
                raises=exceptions.ParamValidationError,
                exception_message="invalid signature_algorithm argument provided",
            ),
            param(
                "auto salt length",
                verify_salt_length="auto",
            ),
            param(
                "hash salt length",
                verify_salt_length="hash",
            ),
            param(
                "invalid string for salt length",
                verify_salt_length="foobar",
                raises=exceptions.ParamValidationError,
                exception_message="invalid salt_length argument provided",
            ),
            param(
                "invalid negative salt length",
                verify_salt_length="-2",
                raises=exceptions.ParamValidationError,
                exception_message="invalid salt_length argument provided",
            ),
            param(
                "valid negative salt length",
                verify_salt_length="-1",
            ),
            param(
                "zero salt length",
                verify_salt_length="0",
            ),
            param(
                "positive salt length",
                verify_salt_length="1",
            ),
        ]
    )
    def test_verify_signed_data(
        self,
        label,
        hash_input="hash this ish",
        hash_algorithm="sha2-256",
        signature_algorithm="pss",
        verify_salt_length=None,
        raises=False,
        exception_message="",
    ):
        hash_input = utils.base64ify(hash_input)
        key_name = "testkey"
        key_type = "ed25519"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        if raises:
            # We want the error, possibly related to verify_salt_length,
            # happen while verifying, not signing.
            sign_salt_length = "auto"
        else:
            sign_salt_length = verify_salt_length
        sign_data_response = self.client.secrets.transit.sign_data(
            name=key_name,
            hash_input=hash_input,
            hash_algorithm="sha2-256",
            signature_algorithm="pss",
            salt_length=sign_salt_length,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_data_response: %s" % sign_data_response)
        signature = sign_data_response["data"]["signature"]
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.verify_signed_data(
                    name=key_name,
                    hash_input=hash_input,
                    signature=signature,
                    hash_algorithm=hash_algorithm,
                    signature_algorithm=signature_algorithm,
                    salt_length=verify_salt_length,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            verify_signed_data_response = (
                self.client.secrets.transit.verify_signed_data(
                    name=key_name,
                    hash_input=hash_input,
                    signature=signature,
                    hash_algorithm=hash_algorithm,
                    signature_algorithm=signature_algorithm,
                    salt_length=verify_salt_length,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            )
            logging.debug(
                "verify_signed_data_response: %s" % verify_signed_data_response
            )
            self.assertTrue(
                expr=verify_signed_data_response["data"]["valid"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "allow_plaintext_backup false",
                allow_plaintext_backup=False,
                raises=exceptions.InternalServerError,
                exception_message="plaintext backup is disallowed on the policy",
            ),
        ]
    )
    @skipIf(
        utils.vault_version_lt("0.9.1"),
        "transit key export/restore added in Vault versions >=0.9.1",
    )
    def test_backup_key(
        self, label, allow_plaintext_backup=True, raises=False, exception_message=""
    ):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        update_key_configuration_response = (
            self.client.secrets.transit.update_key_configuration(
                name=key_name,
                exportable=True,
                allow_plaintext_backup=allow_plaintext_backup,
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "update_key_configuration_response: %s" % update_key_configuration_response
        )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.backup_key(
                    name=key_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            backup_key_response = self.client.secrets.transit.backup_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("backup_key_response: %s" % backup_key_response)
            self.assertIn(
                member="backup",
                container=backup_key_response["data"],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "success with force",
                force=True,
            ),
            param(
                "existing key without force",
                name=None,
                raises=exceptions.InternalServerError,
                exception_message="already exists",
            ),
        ]
    )
    @skipIf(
        utils.vault_version_lt("0.9.1"),
        "transit key export/restore added in Vault versions >=0.9.1",
    )
    def test_restore_key(
        self, label, name="new_test_ky", force=False, raises=False, exception_message=""
    ):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        update_key_configuration_response = (
            self.client.secrets.transit.update_key_configuration(
                name=key_name,
                exportable=True,
                allow_plaintext_backup=True,
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "update_key_configuration_response: %s" % update_key_configuration_response
        )
        backup_key_response = self.client.secrets.transit.backup_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("backup_key_response: %s" % backup_key_response)
        backup = backup_key_response["data"]["backup"]
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.restore_key(
                    backup=backup,
                    name=name,
                    force=force,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            restore_key_response = self.client.secrets.transit.restore_key(
                backup=backup,
                name=name,
                force=force,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("restore_key_response: %s" % restore_key_response)
            self.assertEqual(
                first=bool(restore_key_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    @skipIf(
        utils.vault_version_lt("0.11.4"),
        "transit key trimming added in Vault versions >=0.11.4",
    )
    def test_trim_key(self, label, min_version=2, raises=False, exception_message=""):
        key_name = "testkey"
        create_key_response = self.client.secrets.transit.create_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_key_response: %s" % create_key_response)
        for _ in range(0, 10):
            rotate_key_response = self.client.secrets.transit.rotate_key(
                name=key_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("rotate_key_response: %s" % rotate_key_response)

        update_key_configuration_response = (
            self.client.secrets.transit.update_key_configuration(
                name=key_name,
                min_decryption_version=3,
                min_encryption_version=9,
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "update_key_configuration_response: %s" % update_key_configuration_response
        )

        read_key_response = self.client.secrets.transit.read_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_key_response: %s" % read_key_response)
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.transit.trim_key(
                    name=key_name,
                    min_version=min_version,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            trim_key_response = self.client.secrets.transit.trim_key(
                name=key_name,
                min_version=min_version,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("trim_key_response: %s" % trim_key_response)
            self.assertEqual(
                first=bool(trim_key_response),
                second=True,
            )



import logging
from unittest import TestCase

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestKey(HvacIntegrationTestCase, TestCase):
    def test_start_generate_root_with_completion(self):
        test_otp = utils.get_generate_root_otp()

        self.assertFalse(self.client.sys.read_root_generation_progress()["started"])
        start_generate_root_response = self.client.sys.start_root_token_generation(
            otp=test_otp,
        )
        logging.debug("generate_root_response: %s" % start_generate_root_response)
        self.assertTrue(self.client.sys.read_root_generation_progress()["started"])

        nonce = start_generate_root_response["nonce"]

        last_generate_root_response = {}
        for key in self.manager.keys[0:3]:
            last_generate_root_response = self.client.sys.generate_root(
                key=key,
                nonce=nonce,
            )
        logging.debug("last_generate_root_response: %s" % last_generate_root_response)
        self.assertFalse(self.client.sys.read_root_generation_progress()["started"])

        new_root_token = utils.decode_generated_root_token(
            encoded_token=last_generate_root_response["encoded_root_token"],
            otp=test_otp,
        )
        logging.debug("new_root_token: %s" % new_root_token)
        token_lookup_resp = self.client.lookup_token(token=new_root_token)
        logging.debug("token_lookup_resp: %s" % token_lookup_resp)

        # Assert our new root token is properly formed and authenticated
        self.client.token = new_root_token
        if self.client.is_authenticated():
            self.manager.root_token = new_root_token
        else:
            # If our new token was unable to authenticate, set the test client's token back to the original value
            self.client.token = self.manager.root_token
            self.fail("Unable to authenticate with the newly generated root token.")

    def test_start_generate_root_then_cancel(self):
        test_otp = utils.get_generate_root_otp()

        self.assertFalse(self.client.sys.read_root_generation_progress()["started"])
        self.client.sys.start_root_token_generation(
            otp=test_otp,
        )
        self.assertTrue(self.client.sys.read_root_generation_progress()["started"])

        self.client.sys.cancel_root_generation()
        self.assertFalse(self.client.sys.read_root_generation_progress()["started"])

    def test_rotate(self):
        status = self.client.key_status

        self.client.sys.rotate_encryption_key()

        self.assertGreater(
            self.client.key_status["term"],
            status["term"],
        )

    def test_rekey_multi(self):
        cls = type(self)

        self.assertFalse(self.client.rekey_status["started"])

        self.client.sys.start_rekey()
        self.assertTrue(self.client.rekey_status["started"])

        self.client.sys.cancel_rekey()
        self.assertFalse(self.client.rekey_status["started"])

        result = self.client.sys.start_rekey()

        keys = cls.manager.keys

        result = self.client.sys.rekey_multi(keys, nonce=result["nonce"])
        self.assertTrue(result["complete"])

        cls.manager.keys = result["keys"]
        cls.manager.unseal()

    def test_rekey_verify_multi(self):
        cls = type(self)

        # Start rekey process with verification required and use operator keys
        self.assertFalse(self.client.sys.read_rekey_progress()["started"])
        result = self.client.sys.start_rekey(require_verification=True)
        result = self.client.sys.rekey_multi(cls.manager.keys, nonce=result["nonce"])
        self.assertTrue(result["complete"])
        cls.manager.keys = result["keys"]

        # get the initial verification nonce
        result = self.client.sys.read_rekey_verify_progress()
        first_nonce = result["nonce"]

        # now cancel the process and verify we have a new verification nonce
        result = self.client.sys.cancel_rekey_verify()
        second_nonce = result["nonce"]
        self.assertNotEqual(first_nonce, second_nonce)

        # finally complete the verification process
        result = self.client.sys.rekey_verify_multi(
            cls.manager.keys, nonce=result["nonce"]
        )
        self.assertTrue(result["complete"])

        # now we unseal
        cls.manager.unseal()

    def test_get_backed_up_keys(self):
        with self.assertRaises(exceptions.InvalidRequest) as cm:
            self.client.sys.read_backup_keys()
            self.assertEqual(
                first="no backed-up keys found",
                second=str(cm.exception),
            )



import logging
from unittest import TestCase, skipIf

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(
    utils.vault_version_lt("1.4.0") or not utils.is_enterprise(),
    "Transform secrets engine only supported with Enterprise Vault",
)
class TestTransform(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "transform-integration-test"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="transform",
            path=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.TEST_MOUNT_POINT)
        super().tearDown()

    def test_create_or_update_role(self):
        create_response = self.client.secrets.transform.create_or_update_role(
            name="test_role",
            transformations=[
                "creditcard-fpe",
                "creditcard-masking",
            ],
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    def test_read_role(self):
        transformations = [
            "creditcard-fpe",
            "creditcard-masking",
        ]
        create_response = self.client.secrets.transform.create_or_update_role(
            name="test_role",
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        read_response = self.client.secrets.transform.read_role(
            name="test_role",
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"read_response: {read_response}")
        self.assertEqual(
            first=read_response["data"]["transformations"],
            second=transformations,
        )

    def test_list_roles(self):
        role_name = "test_role"
        transformations = [
            "creditcard-fpe",
            "creditcard-masking",
        ]
        create_response = self.client.secrets.transform.create_or_update_role(
            name=role_name,
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        list_roles_response = self.client.secrets.transform.list_roles(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_roles_response: {list_roles_response}")
        self.assertEqual(
            first=list_roles_response["data"]["keys"],
            second=[role_name],
        )

    def test_delete_role(self):
        role_name = "test_role"
        transformations = [
            "creditcard-fpe",
            "creditcard-masking",
        ]
        create_response = self.client.secrets.transform.create_or_update_role(
            name=role_name,
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        create_response = self.client.secrets.transform.create_or_update_role(
            name="other-role",
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        list_roles_response = self.client.secrets.transform.list_roles(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_roles_response: {list_roles_response}")
        self.assertIn(
            member=role_name,
            container=list_roles_response["data"]["keys"],
        )
        delete_role_response = self.client.secrets.transform.delete_role(
            name=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"delete_role_response: {delete_role_response}")
        self.assertEqual(
            first=delete_role_response.status_code,
            second=204,
        )
        list_roles_response = self.client.secrets.transform.list_roles(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_roles_response: {list_roles_response}")
        self.assertNotIn(
            member=role_name,
            container=list_roles_response["data"]["keys"],
        )

    def test_create_or_update_transformation(self):
        create_response = self.client.secrets.transform.create_or_update_transformation(
            name="test-transformation",
            transform_type="fpe",
            template="builtin/creditcardnumber",
            tweak_source="internal",
            allowed_roles=["test-role"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    @skipIf(
        utils.vault_version_lt("1.7.0"),
        "Expanded tokenization support added in Vault version v1.7.0",
    )
    def test_create_or_update_fpe_transformation(self):
        create_response = (
            self.client.secrets.transform.create_or_update_fpe_transformation(
                name="test-transformation",
                template="builtin/creditcardnumber",
                tweak_source="internal",
                allowed_roles=["test-role"],
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    @skipIf(
        utils.vault_version_lt("1.7.0"),
        "Expanded tokenization support added in Vault version v1.7.0",
    )
    def test_create_or_update_masking_transformation(self):
        create_response = (
            self.client.secrets.transform.create_or_update_masking_transformation(
                name="test-masking",
                template="builtin/creditcardnumber",
                masking_character="-",
                allowed_roles=["test-role"],
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    @skipIf(
        utils.vault_version_lt("1.7.0"),
        "Expanded tokenization support added in Vault version v1.7.0",
    )
    def test_create_or_update_tokenization_transformation(self):
        create_response = (
            self.client.secrets.transform.create_or_update_tokenization_transformation(
                name="test-tokenization",
                max_ttl="60",
                mapping_mode="exportable",
                allowed_roles=["test-role"],
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    def test_read_transformation(self):
        test_name = "test-transformation"
        template = "builtin/creditcardnumber"
        create_response = self.client.secrets.transform.create_or_update_transformation(
            name=test_name,
            transform_type="fpe",
            template=template,
            tweak_source="internal",
            allowed_roles=["test-role"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        read_response = self.client.secrets.transform.read_transformation(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"read_response: {read_response}")
        self.assertIn(
            member=template,
            container=read_response["data"]["templates"],
        )

    def test_list_transformations(self):
        test_name = "test-transformation"
        template = "builtin/creditcardnumber"
        create_response = self.client.secrets.transform.create_or_update_transformation(
            name=test_name,
            transform_type="fpe",
            template=template,
            tweak_source="internal",
            allowed_roles=["test-role"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        list_response = self.client.secrets.transform.list_transformations(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_response: {list_response}")
        self.assertIn(
            member=test_name,
            container=list_response["data"]["keys"],
        )

    def test_delete_transformation(self):
        test_name = "test-transformation"
        template = "builtin/creditcardnumber"
        create_response = self.client.secrets.transform.create_or_update_transformation(
            name=test_name,
            transform_type="fpe",
            template=template,
            tweak_source="internal",
            allowed_roles=["test-role"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        delete_response = self.client.secrets.transform.delete_transformation(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"delete_response: {delete_response}")
        self.assertEqual(
            first=delete_response.status_code,
            second=204,
        )

    def test_create_or_update_template(self):
        test_name = "test-template"
        create_response = self.client.secrets.transform.create_or_update_template(
            name=test_name,
            template_type="regex",
            pattern="(\\d{9})",
            alphabet="builtin/numeric",
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    def test_read_template(self):
        test_name = "test-template"
        test_pattern = "(\\d{9})"
        create_response = self.client.secrets.transform.create_or_update_template(
            name=test_name,
            template_type="regex",
            pattern=test_pattern,
            alphabet="builtin/numeric",
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        read_response = self.client.secrets.transform.read_template(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"read_response: {read_response}")
        self.assertIn(
            member=test_pattern,
            container=read_response["data"]["pattern"],
        )

    def test_list_templates(self):
        test_name = "test-template"
        test_pattern = "(\\d{9})"
        create_response = self.client.secrets.transform.create_or_update_template(
            name=test_name,
            template_type="regex",
            pattern=test_pattern,
            alphabet="builtin/numeric",
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        list_response = self.client.secrets.transform.list_templates(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_response: {list_response}")
        self.assertIn(
            member=test_name,
            container=list_response["data"]["keys"],
        )

    def test_delete_template(self):
        test_name = "test-template"
        test_pattern = "(\\d{9})"
        create_response = self.client.secrets.transform.create_or_update_template(
            name=test_name,
            template_type="regex",
            pattern=test_pattern,
            alphabet="builtin/numeric",
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        delete_response = self.client.secrets.transform.delete_template(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"delete_response: {delete_response}")
        self.assertEqual(
            first=delete_response.status_code,
            second=204,
        )

    def test_create_or_update_alphabet(self):
        test_name = "test-alphabet"
        test_alphabet = "abc"
        create_response = self.client.secrets.transform.create_or_update_alphabet(
            name=test_name,
            alphabet=test_alphabet,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        self.assertEqual(
            first=create_response.status_code,
            second=204,
        )

    def test_read_alphabet(self):
        test_name = "test-alphabet"
        test_alphabet = "abc"
        create_response = self.client.secrets.transform.create_or_update_alphabet(
            name=test_name,
            alphabet=test_alphabet,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        read_response = self.client.secrets.transform.read_alphabet(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"read_response: {read_response}")
        self.assertEqual(
            first=test_alphabet,
            second=read_response["data"]["alphabet"],
        )

    def test_list_alphabets(self):
        test_name = "test-alphabet"
        test_alphabet = "abc"
        create_response = self.client.secrets.transform.create_or_update_alphabet(
            name=test_name,
            alphabet=test_alphabet,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        list_response = self.client.secrets.transform.list_alphabets(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"list_response: {list_response}")
        self.assertIn(
            member=test_name,
            container=list_response["data"]["keys"],
        )

    def test_delete_alphabet(self):
        test_name = "test-alphabet"
        test_alphabet = "abc"
        create_response = self.client.secrets.transform.create_or_update_alphabet(
            name=test_name,
            alphabet=test_alphabet,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_response: {create_response}")
        delete_response = self.client.secrets.transform.delete_alphabet(
            name=test_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"delete_response: {delete_response}")
        self.assertEqual(
            first=delete_response.status_code,
            second=204,
        )

    def test_encode(self):
        role_name = "test-role"
        transformation_name = "test-transformation"
        transformations = [transformation_name]
        test_input_value = "1111-1111-1111-1111"
        expected_output = "****-****-****-****"
        create_role_response = self.client.secrets.transform.create_or_update_role(
            name=role_name,
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_role_response: {create_role_response}")
        create_transform_response = (
            self.client.secrets.transform.create_or_update_transformation(
                name=transformation_name,
                transform_type="masking",
                template="builtin/creditcardnumber",
                tweak_source="internal",
                allowed_roles=[role_name],
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(f"create_transform_response: {create_transform_response}")
        encode_response = self.client.secrets.transform.encode(
            role_name=role_name,
            value=test_input_value,
            transformation=transformation_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"encode_response: {encode_response}")
        self.assertEqual(
            first=encode_response["data"]["encoded_value"],
            second=expected_output,
        )

    def test_decode(self):
        role_name = "test-role"
        transformation_name = "test-transformation"
        transformations = [transformation_name]
        test_input_value = "1111-1111-1111-1111"
        create_role_response = self.client.secrets.transform.create_or_update_role(
            name=role_name,
            transformations=transformations,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"create_role_response: {create_role_response}")
        create_transform_response = (
            self.client.secrets.transform.create_or_update_transformation(
                name=transformation_name,
                transform_type="fpe",
                template="builtin/creditcardnumber",
                tweak_source="internal",
                allowed_roles=[role_name],
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(f"create_transform_response: {create_transform_response}")
        encode_response = self.client.secrets.transform.encode(
            role_name=role_name,
            value=test_input_value,
            transformation=transformation_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"encode_response: {encode_response}")
        decode_response = self.client.secrets.transform.decode(
            role_name=role_name,
            value=encode_response["data"]["encoded_value"],
            transformation=transformation_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(f"decode_response: {decode_response}")
        self.assertEqual(
            first=decode_response["data"]["decoded_value"],
            second=test_input_value,
        )



import json
import logging
from unittest import TestCase, skipIf

from parameterized import parameterized, param

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(
    utils.vault_version_lt("0.9.0"),
    "Policy class uses new parameters added >= Vault 0.9.0",
)
class TestPolicy(HvacIntegrationTestCase, TestCase):
    TEST_POLICY_NAME = "test-policy-policy"

    def tearDown(self):
        self.client.sys.delete_policy(
            name=self.TEST_POLICY_NAME,
        )
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "pretty print false",
                pretty_print=False,
            ),
        ]
    )
    @skipIf(
        utils.vault_version_eq("0.11.0"),
        "Policy parsing broken in Vault version 0.11.0",
    )
    def test_create_or_update_policy(self, label, pretty_print=True):
        test_policy = {
            "path": {
                "test-path": {
                    "capabilities": ["read"],
                },
            },
        }
        create_policy_response = self.client.sys.create_or_update_policy(
            name=self.TEST_POLICY_NAME,
            policy=test_policy,
            pretty_print=pretty_print,
        )
        logging.debug("create_policy_response: %s" % create_policy_response)
        self.assertEqual(
            first=bool(create_policy_response),
            second=True,
        )

        read_policy_response = self.client.sys.read_policy(
            name=self.TEST_POLICY_NAME,
        )
        logging.debug("read_policy_response: %s" % read_policy_response)
        self.assertDictEqual(
            d1=json.loads(read_policy_response["data"]["rules"]),
            d2=test_policy,
        )

    def test_policy_manipulation(self):
        self.assertIn(
            member="root",
            container=self.client.sys.list_policies()["data"]["policies"],
        )
        self.assertIsNone(self.client.get_policy("test"))
        policy, parsed_policy = self.prep_policy("test")
        self.assertIn(
            member="test",
            container=self.client.sys.list_policies()["data"]["policies"],
        )
        self.assertEqual(policy, self.client.sys.read_policy("test")["data"]["rules"])
        self.assertEqual(parsed_policy, self.client.get_policy("test", parse=True))

        self.client.sys.delete_policy(
            name="test",
        )
        self.assertNotIn(
            member="test",
            container=self.client.sys.list_policies()["data"]["policies"],
        )

    def test_json_policy_manipulation(self):
        self.assertIn(
            member="root",
            container=self.client.sys.list_policies()["data"]["policies"],
        )

        policy = """
            path "sys" {
                policy = "deny"
            }
            path "secret" {
                policy = "write"
            }
        """
        self.client.sys.create_or_update_policy(
            name="test",
            policy=policy,
        )
        self.assertIn(
            member="test",
            container=self.client.sys.list_policies()["data"]["policies"],
        )

        self.client.sys.delete_policy("test")
        self.assertNotIn(
            member="test",
            container=self.client.sys.list_policies()["data"]["policies"],
        )



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac import exceptions
from hvac.adapters import JSONAdapter
from hvac.api.auth_methods import AppRole
from hvac.constants.approle import DEFAULT_MOUNT_POINT


class TestAppRole(TestCase):
    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT, "default", None),
            ("custom mount point", "approle-test", "default", None),
            (
                "bad token type",
                DEFAULT_MOUNT_POINT,
                "bad_token",
                exceptions.ParamValidationError,
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_create_or_update_approle(
        self, test_label, mount_point, token_type, raises, requests_mocker
    ):
        expected_status_code = 204
        role_name = "testrole"

        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}".format(
                mount_point=mount_point, role_name=role_name
            )
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )

        app_role = AppRole(adapter=JSONAdapter())

        if raises is not None:
            with self.assertRaises(raises) as cm:
                app_role.create_or_update_approle(
                    role_name=role_name,
                    token_policies=["default"],
                    token_type=token_type,
                    mount_point=mount_point,
                )
            self.assertIn(member="unsupported token_type", container=str(cm.exception))
        else:
            response = app_role.create_or_update_approle(
                role_name=role_name, token_policies=["default"], mount_point=mount_point
            )

            self.assertEqual(first=expected_status_code, second=response.status_code)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_roles(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {"keys": ["testrole"]},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role".format(
            mount_point=mount_point
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.list_roles(mount_point=mount_point)

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_role(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"

        mock_response = {
            "auth": None,
            "data": {
                "bind_secret_id": True,
                "local_secret_ids": False,
                "secret_id_bound_cidrs": None,
                "secret_id_num_uses": 0,
                "secret_id_ttl": 0,
                "token_bound_cidrs": None,
                "token_explicit_max_ttl": 0,
                "token_max_ttl": 0,
                "token_no_default_poolicy": False,
                "token_num_uses": 0,
                "token_period": 14400,
                "token_policies": None,
                "token_ttl": 0,
                "token_type": "default",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}".format(
                mount_point=mount_point, role_name=role_name
            )
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.read_role(role_name="testrole", mount_point=mount_point)

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_role(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        role_name = "testrole"

        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}".format(
                mount_point=mount_point, role_name=role_name
            )
        )
        requests_mocker.register_uri(
            method="DELETE",
            url=mock_url,
            status_code=expected_status_code,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.delete_role(role_name=role_name, mount_point=mount_point)

        self.assertEqual(first=expected_status_code, second=response.status_code)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_role_id(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"

        mock_response = {
            "auth": None,
            "data": {"role_id": "e5a7b66e-5d08-da9c-7075-71984634b882"},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/role-id".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.read_role_id(role_name=role_name, mount_point=mount_point)

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_update_role_id(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"
        role_id = "test_role_id"

        mock_response = {
            "auth": None,
            "data": {"role_id": role_id},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/role-id".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.update_role_id(
            role_name=role_name, role_id=role_id, mount_point=mount_point
        )

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT, None, None, None),
            (
                "metadata as dict",
                DEFAULT_MOUNT_POINT,
                None,
                {"a": "val1", "b": "two"},
                300,
            ),
            (
                "invalid metadata",
                DEFAULT_MOUNT_POINT,
                exceptions.ParamValidationError,
                "bad metadata",
                None,
            ),
            ("custom mount point", "approle-test", None, None, "5m"),
        ]
    )
    @requests_mock.Mocker()
    def test_generate_secret_id(
        self, test_label, mount_point, raises, metadata, wrap_ttl, requests_mocker
    ):
        expected_status_code = 200
        role_name = "testrole"

        mock_response = {
            "auth": None,
            "data": {
                "secret_id": "841771dc-11c9-bbc7-bcac-6a3945a69cd9",
                "secret_id_accessor": "84896a0c-1347-aa90-a4f6-aca8b7558780",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }

        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id".format(
            mount_point=mount_point, role_name=role_name
        )
        adapter = requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())

        if raises is not None:
            with self.assertRaises(raises) as cm:
                app_role.generate_secret_id(
                    role_name=role_name,
                    metadata=metadata,
                    mount_point=mount_point,
                    wrap_ttl=wrap_ttl,
                )
            self.assertIn(
                member="unsupported metadata argument", container=str(cm.exception)
            )
            assert adapter.call_count == 0

        else:
            response = app_role.generate_secret_id(
                role_name=role_name,
                cidr_list=["127.0.0.1/32"],
                mount_point=mount_point,
                metadata=metadata,
                wrap_ttl=wrap_ttl,
            )

            self.assertEqual(first=mock_response, second=response)
            assert adapter.call_count == 1
            last_request = adapter.last_request
            assert ("metadata" in last_request.json()) == (metadata is not None)

            if wrap_ttl is None:
                assert "X-Vault-Wrap-TTL" not in last_request.headers
            else:
                assert "X-Vault-Wrap-TTL" in last_request.headers
                assert last_request.headers["X-Vault-Wrap-TTL"] == str(wrap_ttl)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT, None, None, None),
            (
                "metadata as dict",
                DEFAULT_MOUNT_POINT,
                None,
                {"a": "val1", "b": "two"},
                300,
            ),
            (
                "invalid metadata",
                DEFAULT_MOUNT_POINT,
                exceptions.ParamValidationError,
                "bad metadata",
                None,
            ),
            ("custom mount point", "approle-test", None, None, "5m"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_custom_secret_id(
        self, test_label, mount_point, raises, metadata, wrap_ttl, requests_mocker
    ):
        expected_status_code = 200
        role_name = "testrole"
        secret_id = "custom_secret"

        mock_response = {
            "auth": None,
            "data": {
                "secret_id": secret_id,
                "secret_id_accessor": "84896a0c-1347-aa90-a4f6-aca8b7558780",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/custom-secret-id".format(
            mount_point=mount_point, role_name=role_name
        )
        adapter = requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())

        if raises is not None:
            with self.assertRaises(raises) as cm:
                app_role.create_custom_secret_id(
                    role_name=role_name,
                    secret_id=secret_id,
                    cidr_list=["127.0.0.1/32"],
                    metadata=metadata,
                    mount_point=mount_point,
                    wrap_ttl=wrap_ttl,
                )
            self.assertIn(
                member="unsupported metadata argument", container=str(cm.exception)
            )
            assert adapter.call_count == 0
        else:
            response = app_role.create_custom_secret_id(
                role_name=role_name,
                secret_id=secret_id,
                cidr_list=["127.0.0.1/32"],
                mount_point=mount_point,
                metadata=metadata,
                wrap_ttl=wrap_ttl,
            )

            self.assertEqual(first=mock_response, second=response)
            assert adapter.call_count == 1
            last_request = adapter.last_request
            assert ("metadata" in last_request.json()) == (metadata is not None)

            if wrap_ttl is None:
                assert "X-Vault-Wrap-TTL" not in last_request.headers
            else:
                assert "X-Vault-Wrap-TTL" in last_request.headers
                assert last_request.headers["X-Vault-Wrap-TTL"] == str(wrap_ttl)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_secret_id(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"
        secret_id = "custom_secret"

        mock_response = {
            "auth": None,
            "data": {
                "secret_id": secret_id,
                "secret_id_accessor": "84896a0c-1347-aa90-a4f6-aca8b7558780",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id/lookup".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.read_secret_id(
            role_name=role_name, secret_id=secret_id, mount_point=mount_point
        )

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_destroy_secret_id(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        role_name = "testrole"
        secret_id = "custom_secret"

        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id/destroy".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.destroy_secret_id(
            role_name=role_name, secret_id=secret_id, mount_point=mount_point
        )

        self.assertEqual(first=expected_status_code, second=response.status_code)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_secret_id_accessors(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"

        mock_response = {
            "auth": None,
            "data": {
                "keys": [
                    "ce102d2a-8253-c437-bf9a-aceed4241491",
                    "a1c8dee4-b869-e68d-3520-2040c1a0849a",
                    "be83b7e2-044c-7244-07e1-47560ca1c787",
                    "84896a0c-1347-aa90-a4f6-aca8b7558780",
                    "239b1328-6523-15e7-403a-a48038cdc45a",
                ]
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.list_secret_id_accessors(
            role_name=role_name, mount_point=mount_point
        )

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_secret_id_accessor(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_name = "testrole"
        secret_id = "custom_secret"
        secret_id_accessor = "84896a0c-1347-aa90-a4f6-aca8b7558780"

        mock_response = {
            "auth": None,
            "data": {
                "secret_id": secret_id,
                "secret_id_accessor": "84896a0c-1347-aa90-a4f6-aca8b7558780",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id-accessor/lookup".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.read_secret_id_accessor(
            role_name=role_name,
            secret_id_accessor=secret_id_accessor,
            mount_point=mount_point,
        )

        self.assertEqual(first=mock_response, second=response)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_destroy_secret_id_accessor(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        role_name = "testrole"
        secret_id_accessor = "84896a0c-1347-aa90-a4f6-aca8b7558780"

        mock_url = "http://localhost:8200/v1/auth/{mount_point}/role/{role_name}/secret-id-accessor/destroy".format(
            mount_point=mount_point, role_name=role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.destroy_secret_id_accessor(
            role_name=role_name,
            secret_id_accessor=secret_id_accessor,
            mount_point=mount_point,
        )

        self.assertEqual(first=expected_status_code, second=response.status_code)

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "approle-test"),
        ]
    )
    @requests_mock.Mocker()
    def test_login(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        role_id = "test_role_id"
        secret_id = "custom_secret"

        mock_response = {
            "data": None,
            "auth": {
                "renewable": True,
                "lease_duration": 1200,
                "metadata": None,
                "token_policies": ["default"],
                "accessor": "fd6c9a00-d2dc-3b11-0be5-af7ae0e1d374",
                "client_token": "5b1a0318-679c-9c45-e5c6-d1b9a9035d49",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/login".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )

        app_role = AppRole(adapter=JSONAdapter())
        response = app_role.login(
            role_id=role_id, secret_id=secret_id, mount_point=mount_point
        )

        self.assertEqual(first=mock_response, second=response)
